@@ -1,7 +1,7 @@
 import six
 
 from mangopay.signals import pre_save, post_save
-from mangopay.utils import Money, Address, Birthplace
+from mangopay.utils import Money, Address, Birthplace, InstantPayout
 from . import constants
 from .base import BaseApiModel, BaseApiModelMethods
 from .compat import python_2_unicode_compatible
@@ -13,7 +13,7 @@ from .fields import (PrimaryKeyField, EmailField, CharField,
                      ShippingAddressField, RefundReasonField, ListField, ReportTransactionsFiltersField,
                      ReportWalletsFiltersField, BillingField, SecurityInfoField, PlatformCategorizationField,
                      BirthplaceField, ApplepayPaymentDataField, GooglepayPaymentDataField, ScopeBlockedField,
-                     BrowserInfoField, ShippingField, CurrentStateField, FallbackReasonField)
+                     BrowserInfoField, ShippingField, CurrentStateField, FallbackReasonField, InstantPayoutField)
 from .query import InsertQuery, UpdateQuery, SelectQuery, ActionQuery
 
 
@@ -433,7 +433,7 @@ class Mandate(BaseModel):
 class PayIn(BaseModel):
     credited_user = ForeignKeyField(User, api_name='CreditedUserId', related_name='credited_users')
     credited_funds = MoneyField(api_name='CreditedFunds')
-    credited_wallet = ForeignKeyField(Wallet, api_name='CreditedWalletId', required=True)
+    credited_wallet = ForeignKeyField(Wallet, api_name='CreditedWalletId')
     debited_wallet = ForeignKeyField(Wallet, api_name='DebitedWalletId')
     status = CharField(api_name='Status', choices=constants.STATUS_CHOICES, default=None)
     result_code = CharField(api_name='ResultCode')
@@ -457,8 +457,15 @@ class PayIn(BaseModel):
 
     @classmethod
     def cast(cls, result):
+        if cls.__name__ == "RecurringPayInCIT":
+            return RecurringPayInCIT
+
+        if cls.__name__ == "RecurringPayInMIT":
+            return RecurringPayInMIT
+
         payment_type = result.get('PaymentType')
         execution_type = result.get('ExecutionType')
+
         types = {
             ("CARD", "DIRECT"): DirectPayIn,
             ("CARD", "WEB"): CardWebPayIn,
@@ -470,83 +477,122 @@ class PayIn(BaseModel):
             ("APPLEPAY", "DIRECT"): ApplepayPayIn,
             ("GOOGLEPAY", "DIRECT"): GooglepayPayIn
         }
+
         return types.get((payment_type, execution_type), cls)
 
+
 @python_2_unicode_compatible
-class RecurringPayIn(BaseModel):
-    recurring_payin_registration_id = CharField(api_name='RecurringPayinRegistrationId', required=True)
-    author = ForeignKeyField(User, api_name='AuthorId')
-    creation_date = DateTimeField(api_name='CreationDate')
-    credited_user = ForeignKeyField(User, api_name='CreditedUserId')
+class RecurringPayInRegistration(BaseModel):
+    author = ForeignKeyField(User, api_name='AuthorId', required=True)
+    card = ForeignKeyField(Card, api_name='CardId', required=True)
+    user = ForeignKeyField(User, api_name='CreditedUserId')
     credited_wallet = ForeignKeyField(Wallet, api_name='CreditedWalletId')
-    debited_wallet = ForeignKeyField(Wallet, api_name='DebitedWalletId')
-    card = ForeignKeyField(Card, api_name='CardId')
-    credited_funds = MoneyField(api_name='CreditedFunds')
-    debited_funds = MoneyField(api_name='DebitedFunds')
-    fees = MoneyField(api_name='Fees')
-    status = CharField(api_name='Status')
-    result_code = CharField(api_name='ResultCode')
-    result_message = CharField(api_name='ResultMessage')
-    execution_date = DateTimeField(api_name='ExecutionDate')
-    type = CharField(api_name='Type', choices=constants.TRANSACTION_TYPE_CHOICES, default=None)
-    nature = CharField(api_name='Nature', choices=constants.NATURE_CHOICES, default=None)
-    payment_type = CharField(api_name='PaymentType', choices=constants.PAYOUT_PAYMENT_TYPE, default=None)
-    execution_type = CharField(api_name='ExecutionType', choices=constants.EXECUTION_TYPE_CHOICES, default=None)
-    secure_mode = CharField(api_name='SecureMode',
-                            choices=constants.SECURE_MODE_CHOICES,
-                            default=constants.SECURE_MODE_CHOICES.default)
-    secure_mode_return_url = CharField(api_name='SecureModeReturnURL')
-    secure_mode_redirect_url = CharField(api_name='SecureModeRedirectURL')
-    secure_mode_needed = BooleanField(api_name='SecureModeNeeded')
-    billing = BillingField(api_name='Billing')
-    security_info = SecurityInfoField(api_name='SecurityInfo')
-    culture = CharField(api_name='Culture')
-    ip_address = CharField(api_name='IpAddress')
-    browser_info = BrowserInfoField(api_name='BrowserInfo')
-    shipping = ShippingField(api_name='Shipping')
-    requested_3ds_version = CharField(api_name='Requested3DSVersion')
-    applied_3ds_version = CharField(api_name='Applied3DSVersion')
-    statement_descriptor = CharField(api_name='StatementDescriptor')
+    first_transaction_debited_funds = MoneyField(api_name='FirstTransactionDebitedFunds', required=True)
+    first_transaction_fees = MoneyField(api_name='FirstTransactionFees', required=True)
+    billing = BillingField(api_name='Billing', required=False)
+    shipping = ShippingField(api_name='Shipping', required=False)
+    end_date = DateField(api_name='EndDate')
+    frequency = CharField(api_name='Frequency')
+    fixed_next_amount = BooleanField(api_name='FixedNextAmount')
+    fractioned_payment = BooleanField(api_name='FractionedPayment')
+    migration = BooleanField(api_name='Migration')
+    next_transaction_debited_funds = MoneyField(api_name='NextTransactionDebitedFunds')
+    next_transaction_fees = MoneyField(api_name='NextTransactionFees')
+    free_cycles = IntegerField(api_name='FreeCycles')
+    cycle_number = IntegerField(api_name='CycleNumber')
+    total_amount = IntegerField(api_name='TotalAmount')
+    recurring_type = CharField(api_name='RecurringType')
+    current_state = CurrentStateField(api_name='CurrentState')
+    status = CharField(api_name='Status', choices=constants.STATUS_CHOICES, default=None)
 
     def get_read_only_properties(self):
-        read_only = ["Id", "CreditedUserId", "CreditedFunds", "CreditedWalletId", "DebitedWalletId", "Status",
-                     "ResultCode", "ResultMessage", "ExecutionDate", "Type", "Nature", "PaymentType", "ExecutionType"]
+        read_only = ["Id", "FreeCycles", "CycleNumber", "TotalAmount", "RecurringType", "Status", "CurrentState"]
         return read_only
 
     class Meta:
-        verbose_name = 'base_recurring_payin'
-        verbose_name_plural = 'base_recurring_payins'
+        verbose_name = 'recurring_registration_payin'
+        verbose_name_plural = 'recurring_registration_payins'
+        url = {
+            InsertQuery.identifier: '/recurringpayinregistrations',
+            SelectQuery.identifier: '/recurringpayinregistrations',
+            UpdateQuery.identifier: '/recurringpayinregistrations'
+        }
+
 
 @python_2_unicode_compatible
-class RecurringPayInCIT(RecurringPayIn):
+class RecurringPayInCIT(PayIn):
+    recurring_payin_registration_id = CharField(api_name='RecurringPayinRegistrationId', required=True)
+    browser_info = BrowserInfoField(api_name='BrowserInfo', required=True)
+    ip_address = CharField(api_name='IpAddress', required=True)
+    secure_mode_return_url = CharField(api_name='SecureModeReturnURL', required=True)
+    statement_descriptor = CharField(api_name='StatementDescriptor')
+    debited_funds = MoneyField(api_name='DebitedFunds')
+    fees = MoneyField(api_name='Fees')
+    applied_3ds_version = CharField(api_name='Applied3DSVersion')
+    author = ForeignKeyField(User, api_name='AuthorId')
+    billing = BillingField(api_name='Billing')
+    card = ForeignKeyField(Card, api_name='CardId')
+    creation_date = DateTimeField(api_name='CreationDate')
+    culture = CharField(api_name='Culture')
+    secure_mode_needed = BooleanField(api_name='SecureModeNeeded')
+    secure_mode = CharField(api_name='SecureMode',
+                            choices=constants.SECURE_MODE_CHOICES,
+                            default=constants.SECURE_MODE_CHOICES.default)
+    secure_mode_redirect_url = CharField(api_name='SecureModeRedirectURL')
+    security_info = SecurityInfoField(api_name='SecurityInfo')
+    shipping = ShippingField(api_name='Shipping')
+
+    def get_read_only_properties(self):
+        read_only = ["AuthorId", "Applied3DSVersion", "CardId", "CreationDate", "Culture", "SecureModeNeeded"
+                     , "SecureMode", "SecureModeRedirectURL", "SecurityInfo"]
+        return read_only
+
+    class Meta:
+        verbose_name = 'recurring_payin'
+        verbose_name_plural = 'recurring_payins'
+        url = {
+            InsertQuery.identifier: '/payins/recurring/card/direct',
+            SelectQuery.identifier: '/payins'
+        }
+
+
+@python_2_unicode_compatible
+class RecurringPayInMIT(PayIn):
+    recurring_payin_registration_id = CharField(api_name='RecurringPayinRegistrationId', required=True)
+    debited_funds = MoneyField(api_name='DebitedFunds')
+    fees = MoneyField(api_name='Fees')
+    statement_descriptor = CharField(api_name='StatementDescriptor')
     browser_info = BrowserInfoField(api_name='BrowserInfo')
     ip_address = CharField(api_name='IpAddress')
     secure_mode_return_url = CharField(api_name='SecureModeReturnURL')
-    statement_descriptor = CharField(api_name='StatementDescriptor')
-    tag = CharField(api_name='Tag')
-    debited_funds = MoneyField(api_name='DebitedFunds')
-    fees = MoneyField(api_name='Fees')
+    applied_3ds_version = CharField(api_name='Applied3DSVersion')
+    author = ForeignKeyField(User, api_name='AuthorId')
+    billing = BillingField(api_name='Billing')
+    card = ForeignKeyField(Card, api_name='CardId')
+    creation_date = DateTimeField(api_name='CreationDate')
+    culture = CharField(api_name='Culture')
+    secure_mode_needed = BooleanField(api_name='SecureModeNeeded')
+    secure_mode = CharField(api_name='SecureMode',
+                            choices=constants.SECURE_MODE_CHOICES,
+                            default=constants.SECURE_MODE_CHOICES.default)
+    secure_mode_redirect_url = CharField(api_name='SecureModeRedirectURL')
+    security_info = SecurityInfoField(api_name='SecurityInfo')
+    shipping = ShippingField(api_name='Shipping')
+
+    def get_read_only_properties(self):
+        read_only = ["AuthorId", "Applied3DSVersion", "CardId", "CreationDate", "Culture", "SecureModeNeeded"
+                     , "SecureMode", "SecureModeRedirectURL", "SecurityInfo", "DebitedFunds", "Fees",
+                     "StatementDescriptor", "BrowserInfo", "IpAddress", "SecureModeReturnURL"]
+        return read_only
 
     class Meta:
         verbose_name = 'recurring_payin'
         verbose_name_plural = 'recurring_payins'
         url = {
-            InsertQuery.identifier: '/payins/recurring/card/direct'
+            InsertQuery.identifier: '/payins/recurring/card/direct',
+            SelectQuery.identifier: '/payins'
         }
 
-@python_2_unicode_compatible
-class RecurringPayInMIT(RecurringPayIn):
-    debited_funds = MoneyField(api_name='DebitedFunds')
-    fees = MoneyField(api_name='Fees')
-    statement_descriptor = CharField(api_name='StatementDescriptor')
-    tag = CharField(api_name='Tag')
-
-    class Meta:
-        verbose_name = 'recurring_payin'
-        verbose_name_plural = 'recurring_payins'
-        url = {
-            InsertQuery.identifier: '/payins/recurring/card/direct'
-        }
 
 @python_2_unicode_compatible
 class DirectPayIn(PayIn):
@@ -696,43 +742,6 @@ class GooglepayPayIn(PayIn):
         }
 
 
-class RecurringPayInRegistration(BaseApiModel):
-    id = PrimaryKeyField(api_name='Id')
-    author = ForeignKeyField(User, api_name='AuthorId', required=True)
-    credited_wallet = ForeignKeyField(Wallet, api_name='CreditedWalletId', required=True)
-    card = ForeignKeyField(Card, api_name='CardId', required=True)
-    user = ForeignKeyField(User, api_name='CreditedUserId')
-    first_transaction_debited_funds = MoneyField(api_name='FirstTransactionDebitedFunds', required=True)
-    first_transaction_fees = MoneyField(api_name='FirstTransactionFees', required=True)
-    billing = BillingField(api_name='Billing', required=False)
-    shipping = ShippingField(api_name='Shipping', required=False)
-    end_date = DateField(api_name='EndDate')
-    frequency = CharField(api_name='Frequency')
-    fixed_next_amount = BooleanField(api_name='FixedNextAmount')
-    fractioned_payment = BooleanField(api_name='FractionedPayment')
-    migration = BooleanField(api_name='Migration')
-    next_transaction_debited_funds = MoneyField(api_name='FirstTransactionDebitedFunds')
-    next_transaction_fees = MoneyField(api_name='FirstTransactionFees')
-    free_cycles = IntegerField(api_name='FreeCycles')
-    cycle_number = IntegerField(api_name='CycleNumber')
-    total_amount = IntegerField(api_name='TotalAmount')
-    recurring_type = CharField(api_name='RecurringType')
-    status = CharField(api_name='Status')
-    current_state = CurrentStateField(api_name='CurrentState')
-
-    def get_read_only_properties(self):
-        read_only = ["Id", "FreeCycles", "CycleNumber", "TotalAmount", "RecurringType", "Status", "CurrentState"]
-        return read_only
-
-    class Meta:
-        verbose_name = 'recurring_registration_payin'
-        verbose_name_plural = 'recurring_registration_payins'
-        url = {
-            InsertQuery.identifier: '/recurringpayinregistrations',
-            SelectQuery.identifier: '/recurringpayinregistrations',
-            UpdateQuery.identifier: '/recurringpayinregistrations'
-        }
-
 class CardWebPayIn(PayIn):
     author = ForeignKeyField(User, api_name='AuthorId', required=True)
     credited_wallet = ForeignKeyField(Wallet, api_name='CreditedWalletId', required=True)
@@ -982,6 +991,34 @@ class BankWirePayOut(BaseModel):
     def __str__(self):
         return 'PayOut request from user %s' % self.author_id
 
+@python_2_unicode_compatible
+class PayOutEligibilityResult(BaseModel):
+    instant_payout = InstantPayoutField(api_name='InstantPayout')
+
+
+@python_2_unicode_compatible
+class PayOutEligibility(BaseModel):
+    author = ForeignKeyField(User, api_name='AuthorId', required=True)
+    debited_funds = MoneyField(api_name='DebitedFunds', required=True)
+    bank_account = ForeignKeyField(BankAccount, api_name='BankAccountId', required=True)
+    debited_wallet = ForeignKeyField(Wallet, api_name='DebitedWalletId', required=True)
+    fees = MoneyField(api_name='Fees')
+    payout_mode_requested = CharField(api_name='PayoutModeRequested')
+    bank_wire_ref = CharField(api_name='BankWireRef')
+
+    def check_eligibility(self, **kwargs):
+        insert = InsertQuery(self, **kwargs)
+        insert.identifier = 'PAYOUT_CHECK_ELIGIBILITY'
+        insert.insert_query = self.get_field_dict()
+        return insert.execute(model_klass=PayOutEligibilityResult)
+
+    class Meta:
+        verbose_name = 'payout_eligibility'
+        verbose_name_plural = 'payouts_eligibility'
+        url = {
+            'PAYOUT_CHECK_ELIGIBILITY': '/payouts/reachability'
+        }
+
 
 class Refund(BaseModel):
     author = ForeignKeyField(User, api_name='AuthorId', required=True)
